# -*- coding: utf-8 -*-
#
# Configuration file for the Sphinx documentation builder.
#
# This file does only contain a selection of the most common options. For a
# full list see the documentation:
# http://www.sphinx-doc.org/en/stable/config

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import sys
import io
import os
import subprocess
import sphinx_rtd_theme  # noqa
# sys.path.insert(0, os.path.abspath('.'))

ON_RTD = os.environ.get('READTHEDOCS') == 'True'

# -- Project information -----------------------------------------------------

project = u'djinfo'
copyright = u'2018, Maxime Haineault'
author = u'Maxime Haineault'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.

#
# The short X.Y version.
ROOT_PATH = os.path.dirname(
    os.path.dirname(os.path.abspath(__file__)))

with io.open(os.path.join(ROOT_PATH, 'djinfo/__version__')) as f:
    version = f.readline().strip()

# The full version, including alpha/beta/rc tags.
release = version


# -- General configuration ---------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.ifconfig',
    'sphinx.ext.viewcode',
    'sphinxcontrib.napoleon',
    'sphinx.ext.intersphinx',
    'sphinx.ext.todo',
    # 'sphinx.ext.doctest',
    # 'sphinx.ext.coverage',
]

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = True

# Autodoc settings
autoclass_content = 'both'
autodoc_member_order = 'bysource'
autodoc_default_flags = ['members', 'undoc-members']

# Napoleon settings
napoleon_google_docstring = True
napoleon_numpy_docstring = True

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path .
exclude_patterns = [u'_build', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
# pygments_style = 'sphinx'
highlight_language = 'python'
pygments_style = 'lovelace'


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
# html_theme = 'alabaster'
html_theme = 'sphinx_rtd_theme'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['topics/']

# Custom sidebar templates, must be a dictionary that maps document names
# to template names.
#
# The default sidebars (for documents that don't match any pattern) are
# defined by theme itself.  Builtin themes are using these templates by
# default: ``['localtoc.html', 'relations.html', 'sourcelink.html',
# 'searchbox.html']``.
#
# html_sidebars = {}


# -- Options for HTMLHelp output ---------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'djinfodoc'


# -- Options for LaTeX output ------------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'djinfo.tex', u'djinfo documentation',
     u'Maxime Haineault', 'manual'),
]


# -- Options for manual page output ------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'djinfo', u'djinfo documentation',
     [author], 1)
]


# -- Options for Texinfo output ----------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'djinfo', u'djinfo documentation',
     author, 'djinfo', 'One line description of project.',
     'Miscellaneous'),
]

# Example configuration for intersphinx: refer to the Python standard library.
intersphinx_mapping = {
    'https://docs.python.org/': None,
    'django': ('https://docs.djangoproject.com/en/1.11/', 'https://docs.djangoproject.com/en/1.11/_objects/'),
}


# -- Extension configuration -------------------------------------------------

# def run_apidoc(_):
#     module = 'djinfo'
#     cur_dir = os.path.abspath(os.path.dirname(__file__))
#     output_path = os.path.join(cur_dir, 'apidoc/')
#     cmd_path = 'sphinx-apidoc'
#     if hasattr(sys, 'real_prefix'):  # Check to see if we are in a venv
#         # If we are, assemble the path manually
#         cmd_path = os.path.abspath(
#             os.path.join(sys.prefix, 'bin', 'sphinx-apidoc'))
#     if ON_RTD:
#         module_path = '../{}'.format(module)
#     else:
#         module_path = module
#     subprocess.check_call([
#         cmd_path, '-e', '-o', output_path, module_path, '--force'])
#
#
# def setup(app):
#     app.connect('builder-inited', run_apidoc)



# -*- coding: utf-8 -*-
"""
djinfo generic utils
"""
from __future__ import unicode_literals

import os
import re
import copy
import logging

from datetime import timedelta
from collections import OrderedDict

import django

from djinfo.settings import SETTINGS_MASKS, ENV_MASKS, ENV_EXCLUDE

logger = logging.getLogger(__name__)


def is_masked(k, masks):
    """
    Returns True if provided value should be masked for provided key
    """
    masked = False
    for m in masks:
        if re.search(m, k):
            masked = True
    return masked


def is_excluded(k, excludes):
    """
    Returns True if provided value should be excluded for provided key
    """
    excluded = False
    for e in excludes:
        if re.search(e, k):
            excluded = True
    return excluded


def settings_as_dict():
    """
    Returns django settings as a representable dictionary, filtering
    and masking values
    """
    o = {}
    _settings = copy.deepcopy(
        django.conf.settings.__dict__['_wrapped'].__dict__)
    for k, v in _settings.items():
        if is_masked(k, SETTINGS_MASKS):
            o[k] = '*' * 15
        elif not k.startswith('_'):
            if isinstance(v, timedelta):
                o[k] = str(v)
            elif isinstance(v, (set, tuple)):
                o[k] = list(v)
            else:
                o[k] = v
    return OrderedDict(sorted(o.items(), key=lambda t: t[0]))


def env_as_dict():
    """
    Returns the env as a dict, filtering and masking values
    """
    o = {}
    for k, v in os.environ.items():
        if not is_excluded(k, ENV_EXCLUDE):
            if is_masked(k, ENV_MASKS):
                o[k] = '*' * 15
            else:
                o[k] = v
    return OrderedDict(sorted(o.items(), key=lambda t: t[0]))


def get_context():
    """
    Returns djinfo template context as dictionary
    """
    return {
        'settings': settings_as_dict(),
        'environment': env_as_dict(),
        'django': {
            'version': django.__version__,
        }
    }



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
djinfo rest runner
"""
import os
import sys

# fix sys path so we don't need to setup PYTHONPATH
sys.path.append(os.path.join(os.path.dirname(__file__), "..", ".."))
os.environ['DJANGO_SETTINGS_MODULE'] = 'djinfo.runtests.settings'

import django # pylint: disable=wrong-import-position

if django.VERSION >= (1, 7, 0):
    # starting from 1.7.0 we need to run setup() in order to populate
    # app config
    django.setup()

from django.conf import settings # pylint: disable=wrong-import-position
from django.test.utils import get_runner # pylint: disable=wrong-import-position


def usage():  # pylint: disable=missing-docstring
    return """
    Usage: python runtests.py [app] [-p <pattern>]
    """


def main():  # pylint: disable=missing-docstring
    TestRunner = get_runner(settings)

    # Ugly parameter parsing. We probably want to improve that in future
    # or just use default django test command. This may be problematic,
    # knowing how testing in Django changes from version to version.
    if '-p' in sys.argv:
        try:
            pos = sys.argv.index('-p')
            pattern = sys.argv.pop(pos) and sys.argv.pop(pos)
        except IndexError:
            print(usage())
            sys.exit(1)
    else:
        pattern = None

    test_modules = sys.argv[1:]

    test_runner = TestRunner(verbosity=2, failfast=False, pattern=pattern)

    if len(sys.argv) > 1:
        test_modules = sys.argv[1:]
    elif len(sys.argv) == 1:
        test_modules = []
    else:
        print(usage())
        sys.exit(1)

    if  (1, 6, 0) <= django.VERSION < (1, 9, 0):
        # this is a compat hack because in django>=1.6.0 you must provide
        # module like "djinfo.contrib.plugin" not "plugin"
        from django.db.models import get_app  # pylint: disable=no-name-in-module
        test_modules = [
            # be more strict by adding .tests to not run tests twice
            get_app(module_name).__name__[:-7] + ".tests"
            for module_name
            in test_modules
        ]
    elif django.VERSION >= (1, 9, 0):
        from django.apps import apps  # pylint: disable=no-name-in-module
        test_modules = [
            # be more strict by adding .tests to not run tests twice
            apps.get_app_config(module_name).name + ".tests"
            for module_name
            in test_modules
        ]

    if django.VERSION < (1, 7, 0):
        # starting from 1.7.0 built in django migrations are run
        # for older releases this patch is required to enable testing with
        # migrations
        from south.management.commands import patch_for_test_db_setup  # pylint: disable=import-error
        patch_for_test_db_setup()

    failures = test_runner.run_tests(test_modules or ['djinfo'])
    sys.exit(failures)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
djinfo generic views
"""

from __future__ import unicode_literals

from django.shortcuts import render
from django.views.decorators.http import require_http_methods
from django.http import JsonResponse, HttpResponseBadRequest
from django.contrib.admin.views.decorators import staff_member_required

from djinfo.utils import get_context


@staff_member_required
@require_http_methods(['GET'])
def djinfo(request):  # noqa pylint: disable=inconsistent-return-statements
    """ djinfo process ajax view """
    ctx = get_context()
    if request.is_ajax():
        return HttpResponseBadRequest()
    elif request.GET.get('json'):
        return JsonResponse(ctx)
    return render(request, 'djinfo/index.html', ctx)



# -*- coding: utf-8 -*-
"""
djinfo tests urls
"""
from django.conf.urls import url, include
from django.conf import settings

from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.conf.urls.static import static

from django.contrib import admin
admin.autodiscover()

urlpatterns = [
    url(r'^admin/doc/', include('django.contrib.admindocs.urls')),
    url(r'^admin/', admin.site.urls),
    url(r'^djinfo/', include('djinfo.urls')),
    url(r'^i18n/', include('django.conf.urls.i18n')),
]

# Add media and static files
urlpatterns += staticfiles_urlpatterns()
urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



"""
djinfo settings
"""
from django.conf import settings

SENSIBLE = [
    r'.*PASSPHRASE.*',
    r'.*PASSWORD.*',
    r'.*SECRET.*',
]

SETTINGS_MASKS = getattr(settings, 'DJINFO_MASK_SETTINGS', SENSIBLE)
ENV_MASKS = getattr(settings, 'DJINFO_MASK_ENV', SENSIBLE)
ENV_EXCLUDE = getattr(settings, 'DJINFO_EXCLUDE_ENV', [
    # fzf pollutes the environment with all its
    # settings, we don't really need to see that.
    r'^_fzf.*',
])



# -*- coding: utf-8 -*-
"""
Django tests settings for djinfo demo project.
"""

import os
import django

DEBUG = True

settings_dir = os.path.dirname(__file__)
PROJECT_ROOT = os.path.abspath(settings_dir)

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(PROJECT_ROOT, 'test.db'),
    }
}

# Internationalization
TIME_ZONE = 'America/Montreal'
LANGUAGE_CODE = 'en-us'
ugettext = lambda s: s  # noqa
LANGUAGES = (
    ('en', ugettext('English')),
    ('fr', ugettext('French')),
)
LOCALE_PATHS = (
    os.path.join(PROJECT_ROOT, 'locale'),
)

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale.
USE_L10N = True

# If you set this to False, Django will not use timezone-aware datetimes.
USE_TZ = True
MEDIA_ROOT = os.path.join(PROJECT_ROOT, 'public/media/')
MEDIA_URL = '/media/'
STATIC_ROOT = os.path.join(PROJECT_ROOT, 'public/static/')
STATIC_URL = '/static/'

STATICFILES_DIRS = (
    os.path.join(PROJECT_ROOT, 'demo/static/'),
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = '_g-js)o8z#8=9pr1&amp;05h^1_#)91sbo-)g^(*=-+epxmt4kc9m#'


if django.VERSION < (1, 8):
    TEMPLATE_DEBUG = DEBUG

    TEMPLATE_DIRS = (
        os.path.join(PROJECT_ROOT, 'templates/'),
    )

    # List of callables that know how to import templates from various sources.
    TEMPLATE_LOADERS = (
        'django.template.loaders.filesystem.Loader',
        'django.template.loaders.app_directories.Loader',
    )
else:
    TEMPLATES = [
        {
            'BACKEND': 'django.template.backends.django.DjangoTemplates',
            'DIRS': [
                # insert your TEMPLATE_DIRS here
            ],
            'APP_DIRS': True,
            'OPTIONS': {
                'context_processors': [
                    # Insert your TEMPLATE_CONTEXT_PROCESSORS here or use this
                    # list if you haven't customized them:
                    'django.contrib.auth.context_processors.auth',
                    'django.template.context_processors.debug',
                    'django.template.context_processors.i18n',
                    'django.template.context_processors.media',
                    'django.template.context_processors.static',
                    'django.template.context_processors.tz',
                    'django.contrib.messages.context_processors.messages',
                ],
            },
        },
    ]

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.locale.LocaleMiddleware',
)

if django.VERSION >= (1, 7):
    # Session verification will become mandatory in Django 1.10
    MIDDLEWARE_CLASSES += (
        'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    )

ROOT_URLCONF = 'urls'
WSGI_APPLICATION = 'demo.wsgi.application'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'django.contrib.admindocs',
    'djinfo',
)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}



# -*- coding: utf-8 -*-
"""
EmailHub setup file
"""
import io
import os

from setuptools import setup, find_packages

PACKAGE_NAME, VERSION = ('djinfo', '__version__')
ROOT_PATH = os.path.dirname(os.path.abspath(__file__))
PACKAGE_PATH = os.path.join(ROOT_PATH, PACKAGE_NAME)

with io.open(os.path.join(PACKAGE_PATH, VERSION)) as f:
    VERSION = f.readline().strip()

    with io.open(os.path.join(ROOT_PATH, 'README.rst'), encoding='utf8') as f:
        README = f.read()

        setup(
            name=PACKAGE_NAME,
            version=VERSION,
            description='Django system information page, suitable for production',  # noqa
            long_description=README,
            author='Maxime Haineault',
            author_email='haineault@gmail.com',
            license='MIT',
            url='https://gitlab.com/h3/djinfo',
            packages=find_packages(),
            include_package_data=True,
            zip_safe=True,
            install_requires=[
                'Django>=1.8',
                'six>=1.11.0',
            ],
            extras_require={
                'tests': [
                    'tox>=3.6',
                    'detox>=0.18',
                ],
                'docs': [
                    'Sphinx==1.7.1',
                    'sphinxcontrib-napoleon==0.6.1',
                    'sphinx-rtd-theme==0.4.2',
                    'docutils==0.12',
                ]
            },
            classifiers=[
                'Intended Audience :: Developers',
                'License :: OSI Approved :: MIT License',
                'Operating System :: OS Independent',
                'Programming Language :: Python :: 2',
                'Programming Language :: Python :: 2.7',
                'Programming Language :: Python :: 3',
                'Programming Language :: Python :: 3.4',
                'Programming Language :: Python :: 3.5',
                'Programming Language :: Python :: 3.6',
            ]
        )



# -*- coding: utf-8 -*-
"""
djinfo email backends tests
"""
from django.test import TestCase
from django.contrib.auth import get_user_model

# from djinfo import settings as djinfo_settings

User = get_user_model()


class ConsoleRawEmailBackendsTestCase(TestCase):
    """ djinfo console email backends tests """

    @classmethod
    def setUpTestData(cls):
        cls.user = User.objects.create(
            username='testuser', email='test@user.com', is_active=True,
            is_superuser=True)
        cls.user2 = User.objects.create(
            username='testuser2', email='test2@user.com', is_active=True,
            is_superuser=False)

    def test_tests(self):
        self.assertEqual(True, True)
       #with self.settings(EMAIL_BACKEND=BACKENDS.get('console')):
       #    self.assertEqual(msg.from_email, FROM)



"""
djinfo templatetgs
"""
from django import template
from django.conf import global_settings, settings

register = template.Library()


@register.filter
def type_name(v):
    """
    Returns the type name of a variable (ex: list, str, dict...)
    """
    return str(type(v).__name__).replace('__proxy__', 'unicode')


@register.filter
def is_list(v):
    """
    Returns True if variable is a list
    """
    return isinstance(v, list)


@register.filter
def is_tuple(v):
    """
    Returns True if variable is a tuple
    """
    return isinstance(v, tuple)


@register.filter
def is_set(v):
    """
    Returns True if variable is a set
    """
    return isinstance(v, set)


@register.filter
def is_boolean(v):
    """
    Returns True if variable is a boolean
    """
    return isinstance(v, bool)


@register.filter
def is_dict(v):
    """
    Returns True if variable is a dictionary
    """
    return isinstance(v, dict)


@register.filter
def is_none(v):
    """
    Returns True if variable is None
    """
    return v is None


@register.filter
def is_link(v):
    """
    Returns True if variable is a string that begins as a valid URL
    """
    try:
        return 'http://' in v or 'https://' in v
    except TypeError:
        return False


@register.filter
def is_core(k):
    """
    Returns True if provided django setting name is part of core django
    """
    return hasattr(global_settings, k)


@register.filter
def is_default(k, v):
    """
    Returns True if provided variable is the default value
    """
    if not hasattr(global_settings, k):
        return False
    elif getattr(global_settings, k) == v:
        return True
    elif not is_core(k) and getattr(settings, k) == v:
        return False
    return False



# -*- coding: utf-8 -*-
"""
djinfo URLs
"""
from django.conf.urls import url
from djinfo.views import djinfo

app_name = 'djinfo'  # pylint: disable=C0103
urlpatterns = [  # pylint: disable=C0103
    url(r'^$', djinfo, name='index'),
]
